unit ImageProps;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, PPControl, PPCore, Buttons;

type
  TImagePropsForm = class(TForm)
    GBDestSize: TGroupBox;
    SizeToWidth: TRadioButton;
    SizeToHeight: TRadioButton;
    ESize: TEdit;
    CSize: TUpDown;
    LSizeUnit: TLabel;
    LSize: TLabel;
    GBTransform: TGroupBox;
    TransRotate0: TRadioButton;
    TransRotate90: TRadioButton;
    TransRotate180: TRadioButton;
    TransRotate270: TRadioButton;
    TransFlip: TCheckBox;
    TransMirror: TCheckBox;
    GBColor: TGroupBox;
    CCBrightness: TTrackBar;
    LCBrightness: TLabel;
    LCContrast: TLabel;
    LCGamma: TLabel;
    LCSaturation: TLabel;
    CCContrast: TTrackBar;
    CCGamma: TTrackBar;
    CCSaturation: TTrackBar;
    ECBrightness: TEdit;
    ECContrast: TEdit;
    ECGamma: TEdit;
    ECSaturation: TEdit;
    BClose: TBitBtn;
    LCFactor: TLabel;
    CCFactor: TTrackBar;
    ECFactor: TEdit;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure TransChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SizeChange(Sender: TObject);
    procedure CSizeClick(Sender: TObject; Button: TUDBtnType);
    procedure ESizeKeyPress(Sender: TObject; var Key: Char);
    procedure BCloseClick(Sender: TObject);
    procedure CCBrightnessChange(Sender: TObject);
    procedure LCBrightnessDblClick(Sender: TObject);
    procedure CCContrastChange(Sender: TObject);
    procedure LCContrastClick(Sender: TObject);
    procedure CCGammaChange(Sender: TObject);
    procedure LCGammaClick(Sender: TObject);
    procedure CCSaturationChange(Sender: TObject);
    procedure LCSaturationClick(Sender: TObject);
    procedure CCFactorChange(Sender: TObject);
    procedure LCFactorDblClick(Sender: TObject);
  private
    { Private declarations }
    procedure ColorChange(Sender: TObject);
  public
    { Public declarations }
    procedure SetLSize;
  end;

var
  ImagePropsForm: TImagePropsForm;

implementation

uses Preview;

{$R *.dfm}

procedure TImagePropsForm.FormShow(Sender: TObject);
begin
  PreviewForm.ShowImageProps.Down:=true;
end;

procedure TImagePropsForm.FormHide(Sender: TObject);
begin
  PreviewForm.ShowImageProps.Down:=false;
end;

procedure TImagePropsForm.TransChange(Sender: TObject);
var OldTranspose:boolean; temp:integer;
begin
  with Transforms do begin
    OldTranspose:=Transpose;
    if TransRotate0.Checked then begin
      Transpose:=false;
      Mirror:=TransMirror.Checked;
      Flip:=TransFlip.Checked;
    end else if TransRotate90.Checked then begin
      Transpose:=true;
      Mirror:=not(TransMirror.Checked);
      Flip:=TransFlip.Checked;
    end else if TransRotate180.Checked then begin
      Transpose:=false;
      Mirror:=not(TransMirror.Checked);
      Flip:=not(TransFlip.Checked);
    end else if TransRotate270.Checked then begin
      Transpose:=true;
      Mirror:=TransMirror.Checked;
      Flip:=not(TransFlip.Checked);
    end;
  end;
  if Sender=nil then exit;
  if OldTranspose<>Transforms.Transpose then begin
    temp:=SizeX; SizeX:=SizeY; SizeY:=temp;
    if SizeToWidth.Checked then begin
      SizeToHeight.Checked:=true;
      CSize.Position:=SizeY;
    end else begin
      SizeToWidth.Checked:=true;
      CSize.Position:=SizeX;
    end;
    SetLSize;
  end;
  PPControl.Changed(ctResized);
end;

procedure TImagePropsForm.FormCreate(Sender: TObject);
begin
  TransChange(nil);
  CCBrightnessChange(nil);
  CCContrastChange(nil);
  CCGammaChange(nil);
  CCSaturationChange(nil);
  CCFactorChange(nil);
end;

procedure TImagePropsForm.SetLSize;
begin
  if Assigned(OriginalBMP) then begin
    LSize.Font.Style:=[];
    LSize.Caption:='Bildgre: '+IntToStr(SizeX)+'x'+IntToStr(SizeY)+' px / mm';
  end else begin
    LSize.Font.Style:=[fsItalic];
    LSize.Caption:='[kein Bild]';
  end;
end;

procedure TImagePropsForm.SizeChange(Sender: TObject);
begin
  if SizeToWidth.Checked then CSize.Position:=SizeX else CSize.Position:=SizeY;
end;

procedure TImagePropsForm.CSizeClick(Sender: TObject; Button: TUDBtnType);
begin
  PPControl.Changed(ctOriginal);
end;

procedure TImagePropsForm.ESizeKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then begin
    Key:=#0;
    PPControl.Changed(ctOriginal);
  end;
end;

procedure TImagePropsForm.BCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TImagePropsForm.CCBrightnessChange(Sender: TObject);
var i:integer;
begin
  i:=CCBrightness.Position;
  if i=0 then ECBrightness.Text:=#177#48
  else if i<0 then ECBrightness.Text:=IntToStr(i)
  else ECBrightness.Text:='+'+IntToStr(i);
  ColorChange(Sender);
end;

procedure TImagePropsForm.LCBrightnessDblClick(Sender: TObject);
begin
  CCBrightness.Position:=0;
end;

procedure TImagePropsForm.CCContrastChange(Sender: TObject);
var i:integer;
begin
  i:=CCContrast.Position;
  if i=0 then ECContrast.Text:=#177#48
  else if i<0 then ECContrast.Text:=IntToStr(i)
  else ECContrast.Text:='+'+IntToStr(i);
  ColorChange(Sender);
end;

procedure TImagePropsForm.LCContrastClick(Sender: TObject);
begin
  CCContrast.Position:=0;
end;

procedure TImagePropsForm.CCGammaChange(Sender: TObject);
var s:string[5];
begin
  Str(CCGamma.Position*0.01:0:2,s);
  ECGamma.Text:=s;
  ColorChange(Sender);
end;

procedure TImagePropsForm.LCGammaClick(Sender: TObject);
begin
  CCGamma.Position:=100;
end;

procedure TImagePropsForm.CCSaturationChange(Sender: TObject);
var i:integer;
begin
  i:=CCSaturation.Position;
  if i=0 then ECSaturation.Text:=#177#48
  else if i<0 then ECSaturation.Text:=IntToStr(i)
  else ECSaturation.Text:='+'+IntToStr(i);
  ColorChange(Sender);
end;

procedure TImagePropsForm.LCSaturationClick(Sender: TObject);
begin
  CCSaturation.Position:=0;
end;

procedure TImagePropsForm.ColorChange(Sender: TObject);
begin
  if Sender=nil then exit;
  SetColorRamp(CCFactor.Position,CCBrightness.Position,CCContrast.Position,
               CCGamma.Position,CCSaturation.Position);
  PPControl.Changed(ctResized);
end;

procedure TImagePropsForm.CCFactorChange(Sender: TObject);
var i:integer;
begin
  i:=CCFactor.Position;
  if i=0 then ECFactor.Text:=#177#48
  else if i<0 then ECFactor.Text:=IntToStr(i)
  else ECFactor.Text:='+'+IntToStr(i);
  ColorChange(Sender);
end;

procedure TImagePropsForm.LCFactorDblClick(Sender: TObject);
begin
  CCFactor.Position:=0;
end;

end.
