unit OPTNps;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, PSTools, PSExport, PPControl;

const MARGINUL=8;
      SHADOW=2;
      MARGINLR=MARGINUL+SHADOW;
      SPACING=4;
      FILLCOLOR=$80FFC0;
      LINECOLOR=$408060;
      CROSS=32;

type
  TOPTNpsForm = class(TForm)
    LLineWidth: TLabel;
    LBoxLineWidth: TLabel;
    LInnerLineWidth: TLabel;
    VBoxLineWidth: TLabel;
    VInnerLineWidth: TLabel;
    LColor: TLabel;
    CCenter: TCheckBox;
    CBoxLineWidth: TTrackBar;
    CInnerLineWidth: TTrackBar;
    CColor: TTrackBar;
    PColor: TPanel;
    VColor: TLabel;
    LGray: TLabel;
    CScale: TTrackBar;
    LScale: TLabel;
    VScale: TLabel;
    CLFSize: TTrackBar;
    LLFSize1: TLabel;
    VLFSize: TLabel;
    LLFSize2: TLabel;
    LPageOrder: TLabel;
    PO_Normal: TSpeedButton;
    PO_Reversed: TSpeedButton;
    LayoutInfoBox: TGroupBox;
    LPages: TLabel;
    LBPP: TLabel;
    LBOLP: TLabel;
    LBlocks: TLabel;
    VBOLP: TLabel;
    VBPP: TLabel;
    VPages: TLabel;
    VBlocks: TLabel;
    LPreview: TLabel;
    PPreview: TPanel;
    Preview: TPaintBox;
    BClose: TBitBtn;
    DExportPS: TSaveDialog;
    BExport: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure CScaleChange(Sender: TObject);
    procedure CBoxLineWidthChange(Sender: TObject);
    procedure CInnerLineWidthChange(Sender: TObject);
    procedure CLFSizeChange(Sender: TObject);
    procedure CColorChange(Sender: TObject);
    procedure CCenterClick(Sender: TObject);
    procedure PreviewPaint(Sender: TObject);
    procedure BCloseClick(Sender: TObject);
    procedure OrderChange(Sender: TObject);
    procedure BExportClick(Sender: TObject);
  private
    { Private declarations }
    SIZE:integer;
    procedure LoadSettings;
  public
    { Public declarations }
    procedure LayoutChange;
  end;

var
  OPTNpsForm: TOPTNpsForm;

implementation

{$R *.dfm}

procedure TOPTNpsForm.LoadSettings;
begin
  CScale.Position:=round(PSExScale*100.0);
  CScaleChange(nil);
  CLFSize.Position:=PSExLegendFontSize;
  CLFSizeChange(nil);
  CBoxLineWidth.Position:=round(PSExBoxLineWidth*100.0);
  CBoxLineWidthChange(nil);
  CInnerLineWidth.Position:=round(PSExInnerLineWidth*100.0);
  CInnerLineWidthChange(nil);
  CColor.Position:=round((1.0-PSExGray)*100.0);
  CColorChange(nil);
  CCenter.Checked:=PSExCenter;
  if PSExReverseOrder then PO_Reversed.Down:=true else PO_Normal.Down:=true;
  SIZE:=Preview.Width;
  LayoutChange;
end;

procedure TOPTNpsForm.FormShow(Sender: TObject);
begin
  LoadSettings;
end;

procedure TOPTNpsForm.CScaleChange(Sender: TObject);
begin
  VScale.Caption:=IntToStr(CScale.Position)+'%';
  if Assigned(Sender) then begin
    PSExScale:=CScale.Position*0.01;
    LayoutChange;
  end;
end;

procedure TOPTNpsForm.CBoxLineWidthChange(Sender: TObject);
begin
  VBoxLineWidth.Caption:=Format('%4.2f pt',[CBoxLineWidth.Position*0.01]);
  if Assigned(Sender) then
    PSExBoxLineWidth:=CBoxLineWidth.Position*0.01;
end;

procedure TOPTNpsForm.CInnerLineWidthChange(Sender: TObject);
begin
  VInnerLineWidth.Caption:=Format('%4.2f pt',[CInnerLineWidth.Position*0.01]);
  if Assigned(Sender) then
    PSExInnerLineWidth:=CInnerLineWidth.Position*0.01;
end;

procedure TOPTNpsForm.CLFSizeChange(Sender: TObject);
begin
  VLFSize.Caption:=IntToStr(CLFSize.Position)+' pt';
  if Assigned(Sender) then
    PSExLegendFontSize:=CLFSize.Position;
end;

procedure TOPTNpsForm.CColorChange(Sender: TObject);
var i:integer;
begin
  i:=CColor.Position;
  VColor.Caption:=IntToStr(i)+'%';
  i:=100-i;
  if Assigned(Sender) then PSExGray:=i*0.01;
  if i>50 then PColor.Font.Color:=clBlack else PColor.Font.Color:=clWhite;
  i:=round(i*2.55);
  PColor.Color:=(i SHL 16) OR (i SHL 8) OR i;
end;

procedure TOPTNpsForm.LayoutChange;
begin
  PSExGetLayout;
  with PSExLayout do begin
    VBlocks.Caption:=IntToStr(BlocksX)+' x '+IntToStr(BlocksY);
    VPages.Caption:=IntToStr(PagesX)+' x '+IntToStr(PagesY)+' = '+IntToStr(length(Pages));
    VBPP.Caption:=IntToStr(BPPX)+' x '+IntToStr(BPPY);
    VBOLP.Caption:=IntToStr(BOLPX)+' x '+IntToStr(BOLPY);
  end;
  BExport.Enabled:=(length(PPGrid)>0) AND (PPLegendMax>0);
  Preview.Invalidate;
end;

procedure TOPTNpsForm.CCenterClick(Sender: TObject);
begin
  if Assigned(Sender) then begin
    PSExCenter:=CCenter.Checked;
    LayoutChange;
  end;
end;

procedure TOPTNpsForm.PreviewPaint(Sender: TObject);
var d,PageScale:double;
    BaseX,BaseY:integer;
    PSX,PSY:integer;  // PageSize in Pixeln
    X,Y:integer;
begin with Preview.Canvas,PSExLayout do begin
  Brush.Style:=bsSolid;
  Pen.Style:=psSolid;
  if (length(PPGrid)<=0) OR (PPLegendMax<=0) then begin
    Pen.Width:=10; Pen.Color:=cl3DDkShadow;
    MoveTo(CROSS,CROSS); LineTo(SIZE-CROSS,SIZE-CROSS);
    MoveTo(SIZE-CROSS,CROSS); LineTo(CROSS,SIZE-CROSS);
    exit;
  end else
    Pen.Width:=1;
  PageScale:=PageSizeX/((SIZE-MARGINUL-MARGINLR) DIV PagesX);
  d:=PageSizeY/((SIZE-MARGINUL-MARGINLR) DIV PagesY);
  if d>PageScale then PageScale:=d;
  PSX:=trunc(PageSizeX/PageScale);
  PSY:=trunc(PageSizeY/PageScale);
  BaseX:=(SIZE-SHADOW-PagesX*PSX-(PagesX-1)*SPACING+1) DIV 2;
  BaseY:=(SIZE-SHADOW-PagesY*PSY-(PagesY-1)*SPACING+1) DIV 2;
  Brush.Color:=cl3DDkShadow;
  for Y:=0 to (PagesY-1) do for X:=0 to (PagesX-1) do with Pages[Y*PagesX+X] do begin
    PVX:=BaseX+(PSX+SPACING)*X;
    PVY:=BaseY+(PSY+SPACING)*Y;
    FillRect(Rect(PVX+SHADOW,PVY+SHADOW,PVX+PSX+SHADOW,PVY+PSY+SHADOW));
  end;
  Brush.Color:=clWhite;
  Pen.Color:=clBlack;
  for Y:=0 to (PagesY-1) do for X:=0 to (PagesX-1) do with Pages[Y*PagesX+X] do
    Rectangle(PVX,PVY,PVX+PSX,PVY+PSY);
  Brush.Color:=FILLCOLOR;
  Pen.Color:=LINECOLOR;
  for Y:=0 to (PagesY-1) do for X:=0 to (PagesX-1) do with Pages[Y*PagesX+X] do
    Rectangle(PVX+trunc(X1/PageScale),PVY+trunc(Y1/PageScale),
              PVX+trunc(X2/PageScale),PVY+trunc(Y2/PageScale));                                                
end; end;

procedure TOPTNpsForm.BCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TOPTNpsForm.OrderChange(Sender: TObject);
begin
  if Assigned(Sender) then
    PSExReverseOrder:=PO_Reversed.Down;
end;

procedure TOPTNpsForm.BExportClick(Sender: TObject);
begin
  if (length(PPGrid)<=0) OR (PPLegendMax<=0) then exit;
  if not DExportPS.Execute then exit;
  DoPSExport(DExportPS.FileName);
end;

end.
