unit PPControl;
interface
uses SysUtils, Forms, Dialogs, Graphics, JPEG;

const ColorFile='PaperPic.co3';

type TViewMode=(vmOriginal,vmBase,vmReduced,vmGrid,vmLegend);

var OriginalBMP:TBitmap; SourceAspect:double;
    ResizedBMP:TBitmap;
    BaseBMP:TBitmap;
    ReducedBMP:TBitmap;

var Transforms:record
                 Transpose,Mirror,Flip:boolean;
               end;
var SizeX,SizeY:integer;
var PPData:array of byte;

var PPGrid:array of char;
    PPGPitch,PPGOffset:integer;
type TPPLegendEntry=record Index,Count:integer; end;
var PPLegend:array[byte]of TPPLegendEntry;
    PPLegendMax:integer;
    PPLegendStr:array of string;

function LoadImage(const FileName:string):boolean;

type TChangeType=(ctOriginal,ctResized,ctBase,ctReduced,ctGrid);
procedure Changed(What:TChangeType);

type PRGB=^TRGB;
     TRGB=packed record B,G,R:byte; end;
     PRGBLine=^TRGBLine;
     TRGBLine=array[0..0]of TRGB;
function RGB(R,G,B:integer):TRGB;
function RGB2Color(RGB:TRGB):TColor;
function Color2RGB(Color:TColor):TRGB;

type TPPColorEntryType=(etUnused,etSeparator,etUnselected,etSelected);
     TPPColorName=string[121];
     TPPColor=packed record
                EntryType:TPPColorEntryType;
                Color:TRGB;
                Character:char;
                Groups:byte;
                Name:TPPColorName;
              end;
var PPColors:array[byte]of TPPColor;

procedure LoadColors;
procedure SaveColors;

implementation
uses Preview, PPCore, OPTNps, PSExport;

function LoadImage(const FileName:string):boolean;
var JPEG:TJPEGImage; Success:boolean;
  procedure atexit;
  begin
    if Success then PreviewForm.Caption:='PaperPic III - '+ExtractFileName(FileName)
               else PreviewForm.Caption:='PaperPic III';
    LoadImage:=Success;
    if Assigned(OriginalBMP) then with OriginalBMP do
      SourceAspect:=Width/Height;
    with PreviewForm do
      if ViewMode=vmOriginal then SetViewMode(ViewMode,Zoom);
    Application.ProcessMessages;
    Changed(ctOriginal);
  end;
begin
  Success:=false;
  if (FileName='') OR not(FileExists(FileName)) then begin
    if Assigned(OriginalBMP) then FreeAndNil(OriginalBMP);
    PreviewForm.Caption:='PaperPic III';
    atexit; exit;
  end;
  if not Assigned(OriginalBMP) then OriginalBMP:=TBitmap.Create;
  if UpperCase(ExtractFileExt(FileName))='.BMP' then begin
    try
      OriginalBMP.LoadFromFile(FileName);
      OriginalBMP.PixelFormat:=pf24bit;
    except
      FreeAndNil(OriginalBMP);
      atexit; exit;
    end;
  end else try
    JPEG:=TJPEGImage.Create;
    JPEG.LoadFromFile(FileName);
    OriginalBMP.Assign(JPEG);
    JPEG.Free;
  except
    FreeAndNil(OriginalBMP);
    atexit; exit;
  end;
  Success:=true;
  atexit;
end;

procedure Changed(What:TChangeType);
begin case What of
  ctOriginal:begin
    PPCore_Resize;
    Changed(ctResized);
  end;
  ctResized:begin
    PPCore_Transform;
    Changed(ctBase);
  end;
  ctBase:begin
    with PreviewForm do
      if ViewMode=vmBase then SetViewMode(ViewMode,Zoom);
    PPCore_Reduce;
    Changed(ctReduced);
  end;
  ctReduced:begin
    with PreviewForm do
      if ViewMode=vmReduced then SetViewMode(ViewMode,Zoom);
    PPCore_Finish;
    Changed(ctGrid);
  end;
  ctGrid:begin
    with PreviewForm do
      if ViewMode IN [vmGrid,vmLegend] then SetViewMode(ViewMode,Zoom);
    PSExGetLayout;
    if Assigned(OPTNpsForm) AND OPTNpsForm.Visible then OPTNpsForm.LayoutChange;
  end;
end; end;

function RGB(R,G,B:integer):TRGB;
begin Result.R:=R; Result.G:=G; Result.B:=B; end;

function RGB2Color(RGB:TRGB):TColor;
begin Result:=(RGB.R) OR (RGB.G SHL 8) OR (RGB.B SHL 16); end;

function Color2RGB(Color:TColor):TRGB;
begin Result.R:=Color; Result.G:=Color SHR 8; Result.B:=Color SHR 16; end;

procedure LoadColors;
var t:System.Text; s:string; C:integer;
  function unhex(c:char):integer; begin case c of
    '0'..'9':Result:=byte(c)-48;
    'A'..'Z':Result:=byte(c)-55;
    'a'..'z':Result:=byte(c)-87;
  else Result:=0; end; end;
begin
  for C:=2 to 255 do with PPColors[C] do begin
    EntryType:=etUnused; Color:=RGB(0,0,0); Character:=#32;
    Name[0]:=#0; Groups:=0;
  end;
  with PPColors[0] do begin Name:='Schwarz'; Character:='K'; end;
  with PPColors[1] do begin Name:='Wei'; Character:='.'; end;
  AssignFile(t,ColorFile);
  {$I-} Reset(t); {$I+}
  if IOResult=0 then begin
    for C:=0 to 255 do with PPColors[C] do begin
      if EOF(t) then break;
      readln(t,s);
      if length(s)>0 then begin
        if s[1]='#' then Name:=copy(s,2,length(s)-1)
        else if (length(s)<10) OR not(s[1] IN ['+','-','=']) then Name:=s
        else if s[1]='=' then begin
          EntryType:=etSeparator;
          Name:=copy(s,11,length(s)-10);
        end else begin
          if s[1]='+' then EntryType:=etSelected else EntryType:=etUnselected;
          Color.R:=(unhex(s[2]) SHL 4) OR unhex(s[3]);
          Color.G:=(unhex(s[4]) SHL 4) OR unhex(s[5]);
          Color.B:=(unhex(s[6]) SHL 4) OR unhex(s[7]);
          Groups:=(unhex(s[8]) SHL 4) OR unhex(s[9]);
          Character:=s[10];
          Name:=copy(s,11,length(s)-10);
        end;
      end;
    end;
    CloseFile(t);
  end;
  with PPColors[0] do begin EntryType:=etSelected; Color:=RGB(0,0,0); Groups:=$FF; end;
  with PPColors[1] do begin EntryType:=etSelected; Color:=RGB(255,255,255); Groups:=$FF; end;
end;

procedure SaveColors;
var t:System.Text; s:string;
    LastUsedColor,C:integer;
const Hex:array[0..15]of char='0123456789ABCDEF';
begin
  AssignFile(t,ColorFile);
  {$I-} Rewrite(t); {$I+}
  if IOresult<>0 then begin
    MessageDlg('Kann Farbdatei '+ColorFile+' nicht speichern!',mtError,[mbOK],0);
    exit;
  end;
  LastUsedColor:=255;
  while PPColors[LastUsedColor].EntryType=etUnused do
    dec(LastUsedColor);
  for C:=0 to LastUsedColor do with PPColors[C] do
    if EntryType=etUnused then begin
      if Name[0]=#0 then writeln(t) else writeln(t,'#'+Name);
    end else begin
      s:='========= '+Name;
      if EntryType IN [etSelected,etUnselected] then begin
        if EntryType=etSelected then s[1]:='+' else s[1]:='-';
        s[2]:=Hex[Color.R SHR 4]; s[3]:=Hex[Color.R AND 15];
        s[4]:=Hex[Color.G SHR 4]; s[5]:=Hex[Color.G AND 15];
        s[6]:=Hex[Color.B SHR 4]; s[7]:=Hex[Color.B AND 15];
        s[8]:=Hex[Groups SHR 4]; s[9]:=Hex[Groups AND 15];
        s[10]:=Character;
      end;
      writeln(t,s);
    end;
  CloseFile(t);
end;

initialization
  SetLength(PPGrid,0);
finalization
end.
