unit PageSetup;
interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, PSTools, PPControl;

const in2pt=72.0;        pt2in=1.0/in2pt;
      in2cm=2.54;        cm2in=1.0/in2cm;
      cm2pt=in2pt/in2cm; pt2cm=1.0/cm2pt;

type TFloatPoint=record X,Y:double; end;

type
  TPageSetupForm = class(TForm)
    PaperSizes: TComboBox;
    LUnit: TLabel;
    Unit_pt: TRadioButton;
    Unit_in: TRadioButton;
    Unit_cm: TRadioButton;
    UserSize: TPanel;
    EPSX: TEdit;
    EPSY: TEdit;
    LX: TLabel;
    LUnit1: TLabel;
    Bevel1: TBevel;
    LMX: TLabel;
    LMY: TLabel;
    EMY: TEdit;
    LUnit2: TLabel;
    EMX: TEdit;
    LUnit3: TLabel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Orientation: TPanel;
    CPortrait: TRadioButton;
    CLandscape: TRadioButton;
    BCancel: TBitBtn;
    procedure PaperSizesDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure UnitChange(Sender: TObject);
    procedure PaperSizesChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PageSizeKeyPress(Sender: TObject; var Key: Char);
    procedure OrientationChange(Sender: TObject);
    procedure BCancelClick(Sender: TObject);
  private
    { Private declarations }
    pt2Unit,Unit2pt:double;
    UnitName:string[3];
    function GetCurrentSize:TPoint;
    function GetMargin:TPoint;
    function GetNativeSize(Index:integer):TPoint;
    function UnitSize(const ptSize:TPoint):TFloatPoint;
    function PointSize(const uSize:TFloatPoint):TPoint;
    procedure SetNativeSize(const ptsz,ptmg:TPoint);
    procedure SaveSettings;
  public
    { Public declarations }
  end;

var
  PageSetupForm: TPageSetupForm;

function FloatPoint(X,Y:double):TFloatPoint;
function r10(f:double):string;
procedure EditToFloat(Edit:TEdit;var Dest:double);

implementation

{$R *.dfm}

function FloatPoint(X,Y:double):TFloatPoint; begin Result.X:=X; Result.Y:=Y; end;
function r10(f:double):string; begin r10:=FloatToStr(0.1*round(10.0*f)); end;

procedure EditToFloat(Edit:TEdit;var Dest:double);
begin
  try
    Dest:=StrToFloat(Edit.Text);
  except
  end;
end;

procedure TPageSetupForm.PaperSizesDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var sFormatName:string; p:integer; ptsz:TPoint; usz:TFloatPoint;
begin
  with PaperSizes.Canvas do begin
    FillRect(Rect);
    sFormatName:=PaperSizes.Items[Index];
    p:=pos('|',sFormatName);
    if p>0 then SetLength(sFormatName,p-1);
    ptsz:=GetNativeSize(Index);
    if (ptsz.X>0) AND (ptsz.Y>0) then begin
      usz:=UnitSize(ptsz);
      TextOut(Rect.Left+2,Rect.Top+1,sFormatName+' ('+
              r10(usz.X)+'x'+r10(usz.Y)+#32+UnitName+')');
    end else
      TextOut(Rect.Left+2,Rect.Top+1,sFormatName);
  end;
end;

function TPageSetupForm.GetCurrentSize:TPoint;
var usz:TFloatPoint;
begin
  if PaperSizes.ItemIndex=0 then begin
    usz.X:=0.0; usz.Y:=0.0;
    EditToFloat(EPSX,usz.X);
    EditToFloat(EPSY,usz.Y);
    Result:=PointSize(usz);
  end else
    Result:=GetNativeSize(PaperSizes.ItemIndex);
end;

function TPageSetupForm.GetMargin:TPoint;
var umg:TFloatPoint;
begin
  umg.X:=0.0; umg.Y:=0.0;
  EditToFloat(EMX,umg.X);
  EditToFloat(EMY,umg.Y);
  Result:=PointSize(umg);
end;

function TPageSetupForm.UnitSize(const ptSize:TPoint):TFloatPoint;
begin
  Result.X:=ptSize.X*pt2Unit;
  Result.Y:=ptSize.Y*pt2Unit;
end;

function TPageSetupForm.PointSize(const uSize:TFloatPoint):TPoint;
begin
  Result.X:=round(uSize.X*Unit2pt);
  Result.Y:=round(uSize.Y*Unit2pt);
end;

function TPageSetupForm.GetNativeSize(Index:integer):TPoint;
var s:string; p:integer;
begin
  s:=PaperSizes.Items[Index];
  p:=pos('|',s);
  if p<=0 then begin
    Result.X:=0; Result.Y:=0;
  exit; end;
  s:=copy(s,p+1,length(s)-p);
  p:=pos('x',s);
  if p<=0 then begin
    Result.X:=0; Result.Y:=0;
  exit; end;
  try
    Result.X:=StrToInt(copy(s,1,p-1));
    Result.Y:=StrToInt(copy(s,p+1,length(s)-p));
  except
    Result.X:=0;
    Result.Y:=0;
  end;
end;

procedure TPageSetupForm.UnitChange(Sender: TObject);
var OldSize,OldMargin:TPoint; NewSize,NewMargin:TFloatPoint;
begin
  OldSize:=GetCurrentSize;
  OldMargin:=GetMargin;
  if Unit_pt.Checked then begin
    pt2Unit:=1.0; Unit2pt:=1.0; UnitName:='pt';
  end else if Unit_in.Checked then begin
    pt2Unit:=pt2in; Unit2pt:=in2pt; UnitName:='in';
  end else begin
    pt2Unit:=pt2cm; Unit2pt:=cm2pt; UnitName:='cm';
  end;
  LUnit1.Caption:=UnitName;
  LUnit2.Caption:=UnitName;
  LUnit3.Caption:=UnitName;
  if Assigned(Sender) then begin
    NewMargin:=UnitSize(OldMargin);
    EMX.Text:=r10(NewMargin.X);
    EMY.Text:=r10(NewMargin.Y);
    if PaperSizes.ItemIndex=0 then begin
      NewSize:=UnitSize(OldSize);
      EPSX.Text:=r10(NewSize.X);
      EPSY.Text:=r10(NewSize.Y);
    end;
  end;
  PaperSizes.Invalidate;
  PaperSizesChange(nil);
end;

procedure TPageSetupForm.PaperSizesChange(Sender: TObject);
var Custom:boolean; usz:TFloatPoint;
begin
  Custom:=(PaperSizes.ItemIndex=0);
  EPSX.Enabled:=Custom;
  EPSY.Enabled:=Custom;
  if not Custom then begin
    usz:=UnitSize(GetNativeSize(PaperSizes.ItemIndex));
    EPSX.Text:=r10(usz.X);
    EPSY.Text:=r10(usz.Y);
  end;
  if Assigned(Sender) then SaveSettings;
end;

procedure TPageSetupForm.FormCreate(Sender: TObject);
begin
  UnitChange(nil);
end;

procedure TPageSetupForm.SetNativeSize(const ptsz,ptmg:TPoint);
var i:integer; cmpsz:TPoint; usz:TFloatPoint;
begin
  usz:=UnitSize(ptmg);
  EMX.Text:=r10(usz.X);
  EMY.Text:=r10(usz.Y);
  for i:=1 to (PaperSizes.Items.Count-1) do begin
    cmpsz:=GetNativeSize(i);
    if (ptsz.X=cmpsz.X) AND (ptsz.Y=cmpsz.Y) then begin
      PaperSizes.ItemIndex:=i;
      PaperSizesChange(nil);
      exit;
    end;
  end;
  usz:=UnitSize(ptsz);
  EPSX.Enabled:=true;
  EPSY.Enabled:=true;
  EPSX.Text:=r10(usz.X);
  EPSY.Text:=r10(usz.Y);
  PaperSizes.ItemIndex:=0;
end;

procedure TPageSetupForm.FormShow(Sender: TObject);
begin
  if PageSizeX<PageSizeY then begin
    CPortrait.Checked:=true;
    SetNativeSize(Point(PageSizeX,PageSizeY),Point(MarginX,MarginY));
  end else begin
    CLandscape.Checked:=true;
    SetNativeSize(Point(PageSizeY,PageSizeX),Point(MarginY,MarginX));
  end;
end;

procedure TPageSetupForm.SaveSettings;
var ptsz,ptmg:TPoint;
begin
  ptsz:=GetCurrentSize;
  ptmg:=GetMargin;
  if CLandscape.Checked then begin
    PageSizeX:=ptsz.Y; PageSizeY:=ptsz.X;
    MarginX:=ptmg.Y;   MarginY:=ptmg.X;
  end else begin
    PageSizeX:=ptsz.X; PageSizeY:=ptsz.Y;
    MarginX:=ptmg.X;   MarginY:=ptmg.Y;
  end;
  PPControl.Changed(ctGrid);
end;

procedure TPageSetupForm.PageSizeKeyPress(Sender: TObject; var Key: Char);
begin
  if Key<>#13 then exit;
  Key:=#0;
  SaveSettings;
end;

procedure TPageSetupForm.OrientationChange(Sender: TObject);
begin
  SaveSettings;
end;

procedure TPageSetupForm.BCancelClick(Sender: TObject);
begin
  Close;
end;

end.
