unit PickColor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, PPControl;

type
  TPickColorForm = class(TForm)
    PixBox: TPaintBox;
    OpenPicture: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PixBoxPaint(Sender: TObject);
    procedure PixBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PixBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PixBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    BMP:TBitmap;
    HaveRect:boolean;
    X1,Y1,X2,Y2:integer;
    procedure DoIt;
  public
    { Public declarations }
    function SelectFile:boolean;
  end;

var
  PickColorForm: TPickColorForm;

implementation

uses ColorEdit;

{$R *.dfm}

procedure TPickColorForm.FormCreate(Sender: TObject);
begin
  BMP:=nil;
end;

procedure TPickColorForm.FormDestroy(Sender: TObject);
begin
  FreeAndNil(BMP);
end;

procedure TPickColorForm.FormShow(Sender: TObject);
begin
  if Assigned(BMP) then exit;
  HaveRect:=false;
end;

function TPickColorForm.SelectFile:boolean;
begin
  Result:=OpenPicture.Execute;
  if not Result then exit;
  if not Assigned(BMP) then BMP:=TBitmap.Create;
  BMP.LoadFromFile(OpenPicture.FileName);
  BMP.PixelFormat:=pf24bit;
  PixBox.Width:=BMP.Width;
  PixBox.Height:=BMP.Height;
  ClientWidth:=BMP.Width;
  ClientHeight:=BMP.Height;
end;

procedure TPickColorForm.PixBoxPaint(Sender: TObject);
begin
  PixBox.Canvas.Draw(0,0,BMP);
end;

procedure TPickColorForm.PixBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button=mbRight then if SelectFile then Invalidate;
  if Button<>mbLeft then exit;
  PixBox.Canvas.Pen.Mode:=pmNot;
  PixBox.Canvas.Brush.Style:=bsClear;
  X1:=X; Y1:=Y;
  HaveRect:=false;
end;

procedure TPickColorForm.PixBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if not(ssLeft IN Shift) then exit;
  if HaveRect then PixBox.Canvas.Rectangle(X1,Y1,X2,Y2);
  X2:=X; Y2:=Y;
  if X2>BMP.Width then X2:=BMP.Width;    if X2<0 then X2:=0;
  if Y2>BMP.Height then Y2:=BMP.Height;  if Y2<0 then Y2:=0;
  PixBox.Canvas.Rectangle(X1,Y1,X2,Y2);
  HaveRect:=true;
end;

procedure TPickColorForm.PixBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button<>mbLeft then exit;
  if HaveRect then PixBox.Canvas.Rectangle(X1,Y1,X2,Y2);
  HaveRect:=false;
  DoIt; Hide;
end;

procedure TPickColorForm.DoIt;
var MX1,MY1,MX2,MY2,XLen,X,Y,RSum,GSum,BSum,Count:integer;
    Line:PRGBLine; V:PRGB;
begin
  if X1<X2 then begin MX1:=X1; MX2:=X2-1; end else begin MX1:=X2; MX2:=X1-1; end;
  if Y1<Y2 then begin MY1:=Y1; MY2:=Y2-1; end else begin MY1:=Y2; MY2:=Y1-1; end;
  XLen:=MX2-MX1;
  RSum:=0; GSum:=0; BSum:=0; Count:=0;
  for Y:=MY1 to MY2 do begin
    Line:=BMP.ScanLine[Y];
    V:=@Line[MX1];
    for X:=0 to XLen do begin
      inc(BSum,V^.B);
      inc(GSum,V^.G);
      inc(RSum,V^.R);
      inc(V); inc(Count);
    end;
  end;
  ColorEditForm.PColor.Color:=RGB2Color(RGB(
    round(RSum/Count),round(GSum/Count),round(BSum/Count)));
end;

end.
